/* FGGE - Fabiogiopla Game Engine
 * Licensed under MIT license.
 * http://fabiogiopla.altervista.org/?cat=software/dev_utils/fgge
 * Copyright (c) Fabio Iotti 2012. Some rights reserved.
 */

/** @mainpage Fabiogiopla HTML5 Game Engine Documentation
  * Fabiogiopla HTML5 Game Engine (FGGE) is a Framework projected to develop games in HTML5, for both desktop and mobile devices.<br><br>
  *
  * FGGE is light and simple, it provides functions to manage preloading, graphics, media and saving, also networking will soon be implemented.<br>
  * FGGE is Open Source, you can request code at any time by mailing me at fabio.iotti@mail.com, the whole engine is coded in JavaScript fully supported by Chrome, Firefox and Safari.<br><br>
  *
  * The idea is that the programmer shouldn't care about low level procedures, the goal is to provide a framework to code Cross-Platform, Cross-Browser HTML5 videogames with ease.<br>
  * Documentation is always up-to-date, easy to browse, and guided with examples.<br><br>
  *
  * Thanks for downloading FGGE, I hope you will like it!<br>
  * Don't hesitate to contact me if you have any doubt or to report a bug.<br>
  * Fabio Iotti, The developer.
  */

/// Main namespace for FGGE.
/** This namespace contains all FGGE classes and functions and it is accessible from anywhere. */
namespace GameEngine
{
	/// Rendering tools.
	/** Everything graphics-related is here. */
	namespace Renderer
	{
		/// A renderizable model.
		/** A Model is associated to an image it has a position, a rotation, a transparency and a scale.<br>
		  * It can be rendered at any time.
		  */
		public class Model
		{
			/// Coordinate X.
			/** The X coordinate on the screen of the model (from center point). */
			public var x;
			
			/// Coordinate Y.
			/** The Y coordinate on the screen of the model (from center point). */
			public var y;
			
			/// Transparency.
			/** Between 0 (Completely hidden) and 1 (Fully visible). */
			public var transparency;
			
			/// Rotation
			/** Rotation of the model in radians. */
			public var rotation;
			
			/// Scale X.
			/** This is the ratio of model image width with model size on the screen. */
			public var scaleX;
			
			/// Scale Y.
			/** This is the ratio of model image height with model size on the screen. */
			public var scaleY;
			
			/// Create a Model from path.
			/** Create a new Model, path is the url of an image or the image in base64 notation.<br>
			  * Generate models before starting game with GameEngine.startGame() to prevent lag.
			  *
			  * @code
			  * var testModel1 = new GameEngine.Renderer.Model("test1.png");
			  * @endcode
			  *
			  * @param path The path or base64 notation of an image.
			  */
			public Model(var path) {}
			
			/// Render this model.
			/** Make model be rendered instantly.
			  *
			  * @code
			  * var ball = new GameEngine.Renderer.Model("ball.png");
			  * var time = 0; 
			  *
			  * function gameFunction() {
			  * 	GameEngine.Renderer.clearScreen();
			  * 
			  *		ball.x = Math.sin(time);
			  *		ball.y = Math.cos(time);
			  * 	ball.render();
			  *	
			  *		time++;
			  * }
			  * 
			  * GameEngine.startGame(gameFunction);
			  * @endcode
			  */
			public render() {}
		}
	
		/// The HTML5 canvas.
		/** Everything is rendered to this canvas. */
		public var canvas;
		
		/// The HTML5 2D context.
		/** Everything that is in 2D and has to be rendered should pass from this.<br>
		  * You can render geometrical shapes from this.
		  */
		public var context2D;
		
		/// The HTML5 WebGL context (if available).
		/** Some browsers may support hardware accelerated rendering, if a context of this kind is available, it will appear here.
		  */
		public var contextGL;
		
		/// Generate HTML5 canvas.
		/** Generate a new HTML5 and place it after the last loaded script in the page.<br>
		  * Call this function or GameEngine.Renderer.setCanvas() once, before starting game, starting game without a canvas will make engine crash.
		  */
		public generateCanvas() {}
		
		/// Set HTML5 canvas.
		/** Set the canvas that will be used to render the scene.<br>
		  * Call this function or GameEngine.Renderer.generateCanvas() once, before starting game, starting game without a canvas will make engine crash.
		  *
		  * @code
		  * var gameCanvas = document.createElement("canvas");
		  * document.body.appendChild(gameCanvas);
		  *
		  * GameEngine.Renderer.setCanvas(gameCanvas);
		  * @endcode
		  *
		  * @code
		  * var gameCanvas = document.getElementById("mygame_canvas");
		  *
		  * GameEngine.Renderer.setCanvas(gameCanvas);
		  * @endcode
		  *
		  * @param canvasElement The canvas to use.
		  */
		public setCanvas(var canvasElement) {}
		
		/// Clear the canvas.
		/** Everything rendered in the canvas gets removed after calling this function.<br>
		  * Depending on what your game does, you may want to call this at the start of every frame.
		  */
		public clearScreen() {}
		
		/// Change color of the canvas.
		/** Changes background color as specified.<br>
		  * Specified color has to be in HTML notation.
		  *
		  * @code
		  * // Make background red.
		  * GameEngine.Renderer.setBackgroundColor("#FF0000");
		  * @endcode
		  *
		  * @code
		  * // Make background yellow.
		  * GameEngine.Renderer.setBackgroundColor("yellow");
		  * @endcode
		  *
		  * @param color The new color of the canvas.
		  */
		public setBackgroundColor(var color) {}
		
		/// Enable hardware acceleration if available.
		/** Enabling hardware acceleration may improve speed a little.
		  */
		public setHardwareEnabled(var enabled) {}
		
		/// Resize the canvas.
		/** Change the size of the canvas as specified, also works in realtime.
		  *
		  * @code
		  * GameEngine.Renderer.setCanvasSize(800, 600);
		  * @endcode
		  *
		  * @param width The new width of the canvas in pixels.
		  * @param height The new height of the canvas in pixels.
		  */
		public setCanvasSize(var width, var height) {}
		
		/// Enter or exit fullscreen mode.
		/** WARNING: This is an HTML5 feature under discussion, may not work or produce unexpected behaviours! Do not use in serious projects! <br>
		  * Enter or exit fullscreen mode.
		  *
		  * @param fullscreen Enter fullscreen if true, exit if false.
		  */
		public setFullscreen(var fullscreen) {}
		
		/// Change framerate.
		/** Change the framerate of the game.<br>
		  * NOTE: If the client computer is too slow, framerate will decrease without notice!
		  *
		  * @param fps Frames per second.
		  */
		public setFramerate(var fps) {}
	}
	
	/// Sound related tools.
	/** Everything sound-related is here. */
	namespace Media
	{
		/// Audio object.
		/** This can be a music or sound effect.<br>
		  * This is similar to HTML5 audio, but load-managed.
		  */
		public class Audio
		{
			/// Create an Audio from path.
			/** Create a new Audio, path is the url of an audio file or the audio in base64 notation.
			  *
			  * @code
			  * var explosion = new GameEngine.Media.Audio("explosion.ogg");
			  * @endcode
			  *
			  * @param path The path or base64 notation of an audio file.
			  */
			public Audio(var path) {}
			
			/// Start reproducing.
			/** Start reproducing the Audio. */
			public play() {}
			
			/// Suspend reproduction.
			/** Stop reproducind the Audio without rewinding. */
			public pause() {}
			
			/// Stop reproducing.
			/** Stop reproducing the Audio and rewind. */
			public stop() {}
		}
	}
	
	/// Manage input from keyboard.
	/** Keyboard-related tools are inside this package. */
	namespace Keyboard
	{
		/// Describes a Keyboard event.
		/** Describes a Keyboard event like key pressing or releasing. */
		public class KeyboardEvent
		{
			/// Key was pressed.
			public const EVENT_PRESS;
			
			/// Key was released.
			public const EVENT_RELEASE;
			
			public const KEY_BACKSPACE;
			public const KEY_TAB;
			public const KEY_ENTER;
			public const KEY_SHIFT;
			public const KEY_CTRL;
			public const KEY_ALT;
			public const KEY_PAUSE;
			public const KEY_ESCAPE;
			public const KEY_SPACE;
			public const KEY_PAGEUP;
			public const KEY_PAGEDOWN;
			public const KEY_END;
			public const KEY_HOME;
			public const KEY_LEFT;
			public const KEY_UP;
			public const KEY_RIGHT;
			public const KEY_DOWN;
			public const KEY_PRINT;
			public const KEY_INSERT;
			public const KEY_DELETE;
			public const KEY_0;
			public const KEY_1;
			public const KEY_2;
			public const KEY_3;
			public const KEY_4;
			public const KEY_5;
			public const KEY_6;
			public const KEY_7;
			public const KEY_8;
			public const KEY_9;
			public const KEY_A;
			public const KEY_B;
			public const KEY_C;
			public const KEY_D;
			public const KEY_E;
			public const KEY_F;
			public const KEY_G;
			public const KEY_H;
			public const KEY_I;
			public const KEY_J;
			public const KEY_K;
			public const KEY_L;
			public const KEY_M;
			public const KEY_N;
			public const KEY_O;
			public const KEY_P;
			public const KEY_Q;
			public const KEY_R;
			public const KEY_S;
			public const KEY_T;
			public const KEY_U;
			public const KEY_V;
			public const KEY_W;
			public const KEY_X;
			public const KEY_Y;
			public const KEY_Z;
			public const KEY_NUMPAD0;
			public const KEY_NUMPAD1;
			public const KEY_NUMPAD2;
			public const KEY_NUMPAD3;
			public const KEY_NUMPAD4;
			public const KEY_NUMPAD5;
			public const KEY_NUMPAD6;
			public const KEY_NUMPAD7;
			public const KEY_NUMPAD8;
			public const KEY_NUMPAD9;
			public const KEY_F1;
			public const KEY_F2;
			public const KEY_F3;
			public const KEY_F4;
			public const KEY_F5;
			public const KEY_F6;
			public const KEY_F7;
			public const KEY_F8;
			public const KEY_F9;
			public const KEY_F10;
			public const KEY_F11;
			public const KEY_F12;
			public const KEY_NUMLOCK;
			public const KEY_SCROLLLOCK;
			
			/// Get key number.
			/** Returns the number of the key intercepted with the event.
			  *
			  * @code
			  * // Print ID of intercepted keys.
			  * function keyboardListener(event) {
			  * 	var pressed = (event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS);
			  * 	alert("Key '"+event.getKeyNumber()+"' has been "+(pressed?"pressed":"released")+".");
			  * }
			  *
			  * GameEngine.Keyboard.addKeyboardListener(keyboardListener);
			  * @endcode
			  *
			  * @return The ID of the intercepted key.
			  */
			public var getKeyNumber() {}
			
			/// Detect if a key is pressed or released.
			/** May return EVENT_PRESS or EVENT_RELEASE.
			  *
			  * @code
			  * // Print ID of intercepted keys.
			  * function keyboardListener(event) {
			  * 	var pressed = (event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS);
			  * 	alert("Key '"+event.getKeyNumber()+"' has been "+(pressed?"pressed":"released")+".");
			  * }
			  *
			  * GameEngine.Keyboard.addKeyboardListener(keyboardListener);
			  * @endcode
			  *
			  * @return Key status, can be EVENT_PRESS or EVENT_RELEASE.
			  */
			public var getEventType() {}
		}
	
		/// Add a Keyboard listener.
		/** Start listening for key events.
		  *
		  * @code
		  * // Print ID of intercepted keys.
		  * function keyboardListener(event) {
		  * 	var pressed = (event.getEventType() == GameEngine.Keyboard.KeyboardEvent.EVENT_PRESS);
		  * 	alert("Key '"+event.getKeyNumber()+"' has been "+(pressed?"pressed":"released")+".");
		  * }
		  *
		  * GameEngine.Keyboard.addKeyboardListener(keyboardListener);
		  * @endcode
		  *
		  * @param listener The listener function.
		  */
		public addKeyboardListener(var listener) {}
		
		/// Remove a Keyboard listener.
		/** Remove a previously added Keyboard listener, if the listener doesn't exist, nothing happens.
		  *
		  * @param listener The listener function to remove.
		  */
		public removeKeyboardListener(var listener) {}
	}
	
	/// Manage input from mouse.
	/** Mouse-related tools are inside this package. */
	namespace Mouse
	{
		/// Describes a Mouse event.
		/** Describes a Mouse event like button pressing or releasing, mouse move or wheel rotation. */
		public class MouseEvent
		{
			/// Mouse button pressed.
			public const EVENT_PRESS;
			
			/// Mouse button released.
			public const EVENT_RELEASE;
			
			/// Mouse moved.
			public const EVENT_MOVE;
			
			/// Wheel rotated.
			public const EVENT_WHEEL;
			
			/// Left mouse button.
			public const MOUSE_LEFT;
			
			/// Right mouse button.
			public const MOUSE_RIGHT;
			
			/// Middle mouse button.
			public const MOUSE_MIDDLE;
			
			/// Get mouse button number.
			/** Returns the number of the button intercepted with the event.
			  *
			  * @code
			  * function mouseListener(event) {
			  * 	if(event.getEventType() == GameEngine.Mouse.MouseEvent.EVENT_RELEASE) {
			  * 		if(event.getButtonNumber() == GameEngine.Mouse.MouseEvent.MOUSE_LEFT) {
			  * 			alert("Left mouse button released!");
			  * 		}
			  * 	}
			  * }
			  * 
			  * GameEngine.Mouse.addMouseListener(mouseListener);
			  * @endcode
			  *
			  * @return The ID of the intercepted mouse button, can be MOUSE_LEFT, MOUSE_RIGHT, MOUSE_MIDDLE or a number (one for every mouse button).
			  */
			public var getButtonNumber() {}
			
			/// Get wheel rotation value.
			/** Get wheel rotation value for EVENT_WHEEL events.
			  *
			  * @return The wheel rotation value.
			  */
			public var getWheelValue() {}
			
			/// Detect mouse events.
			/** May return EVENT_PRESS, EVENT_RELEASE, EVENT_MOVE or EVENT_WHEEL.
			  *
			  * @code
			  * function mouseListener(event) {
			  * 	if(event.getEventType() == GameEngine.Mouse.MouseEvent.EVENT_RELEASE) {
			  * 		if(event.getButtonNumber() == GameEngine.Mouse.MouseEvent.MOUSE_LEFT) {
			  * 			alert("Left mouse button released!");
			  * 		}
			  * 	}
			  * }
			  * 
			  * GameEngine.Mouse.addMouseListener(mouseListener);
			  * @endcode
			  *
			  * @return Mouse event, can be EVENT_PRESS, EVENT_RELEASE, EVENT_MOVE or EVENT_WHEEL.
			  */
			public var getEventType() {}
		}
	
		/// Position X of the cursor.
		/** Updated automatically every frame, may produce wrong results if used in a strangely formatted page. */
		public var x;
		
		/// Position Y of the cursor.
		/** Updated automatically every frame, may produce wrong results if used in a strangely formatted page. */
		public var y;
		
		/// Add a Mouse listener.
		/** Start listening for mouse events.
		  *
		  * @code
		  * function mouseListener(event) {
		  * 	if(event.getEventType() == GameEngine.Mouse.MouseEvent.EVENT_RELEASE) {
		  * 		if(event.getButtonNumber() == GameEngine.Mouse.MouseEvent.MOUSE_LEFT) {
		  * 			alert("Left mouse button released!");
		  * 		}
		  * 	}
		  * }
		  * 
		  * GameEngine.Mouse.addMouseListener(mouseListener);
		  * @endcode
		  *
		  * @param listener The listener function.
		  */
		public addMouseListener(var listener) {}
		
		/// Remove a Mouse listener.
		/** Remove a previously added Mouse listener, if the listener doesn't exist, nothing happens.
		  *
		  * @param listener The listener function to remove.
		  */
		public removeMouseListener(var listener) {}
	}
	
	/// Manage input from touchscreen on touch-enabled devices.
	/** Touchscreen-related tools are inside this package.<br>
	  * You may only use these tools to develop touch-based and mobile applications.
	  */
	namespace Touch
	{
		/// Describes a Touch event.
		/** Describes a touch event like a finger touch, a finger removed, and drag movement. */
		public class TouchEvent
		{
			/// Touch started.
			public const EVENT_START;
			
			/// Touch ended.
			public const EVENT_END;
			
			/// Touch changed position.
			public const EVENT_MOVE;
		
			/// Position X of the touch-point.
			/** May produce wrong results if used in a strangely formatted page. */
			public var x;
			
			/// Position Y of the touch-point.
			/** May produce wrong results if used in a strangely formatted page. */
			public var y;
			
			/// Get touch identifier.
			/** Returns the identifier of the touch intercepted with the event.
			  *
			  * @code
			  * function touchListener(event) {
			  * 	if(event.getEventType() == GameEngine.Touch.TouchEvent.EVENT_END) {
			  * 		alert("Touch #" + getTouchID() + " just released!");
			  * 	}
			  * }
			  * 
			  * GameEngine.Touch.addTouchListener(touchListener);
			  * @endcode
			  *
			  * @return The ID of the intercepted touch.
			  */
			public var getTouchID() {}
			
			/// Detect touch events.
			/** May return EVENT_START, EVENT_END or EVENT_MOVE.
			  *
			  * @code
			  * function touchListener(event) {
			  * 	if(event.getEventType() == GameEngine.Touch.TouchEvent.EVENT_END) {
			  * 		alert("Touch just released!");
			  * 	}
			  * }
			  * 
			  * GameEngine.Touch.addTouchListener(touchListener);
			  * @endcode
			  *
			  * @return Mouse event, can be EVENT_START, EVENT_END or EVENT_MOVE.
			  */
			public var getEventType() {}
		}
	
		/// Add a Touch listener.
		/** Start listening for touch events.
		  *
		  * @code
		  * function touchListener(event) {
		  * 	if(event.getEventType() == GameEngine.Touch.TouchEvent.EVENT_END) {
		  * 		alert("Touch just released!");
		  * 	}
		  * }
		  * 
		  * GameEngine.Touch.addTouchListener(touchListener);
		  * @endcode
		  *
		  * @param listener The listener function.
		  */
		public addTouchListener(var listener) {}
		
		/// Remove a Touch listener.
		/** Remove a previously added Touch listener, if the listener doesn't exist, nothing happens.
		  *
		  * @param listener The listener function to remove.
		  */
		public removeTouchListener(var listener) {}
	}
	
	/// Saving/Loading tools.
	/** Wrapper for HTML5 Save/Load functions. */
	namespace Saving
	{
		/// Save something offline.
		/** Data is saved as a string.
		  *
		  * @code
		  * GameEngine.Saving.save("energy", 54);
		  * @endcode
		  *
		  * @param name The name of the saved variable.
		  * @param value The value of the saved variable.
		  */
		public save(var name, var value) {}
		
		/// Load something previously saved.
		/** Data is loaded as string, you can use parseFloat(), parseInt() or eval() to convert data to other formats.
		  * 
		  * @code
		  * GameEngine.Saving.save("executionTimes", parseInt(GameEngine.Saving.load("executionTimes"))+1);
		  * alert("Executed "+GameEngine.Saving.load("executionTimes")+" times...");
		  * @endcode
		  *
		  * @param name The name of the loaded variable.
		  *
		  * @return The value of the loaded variable.
		  */
		public var load(var name) {}
	}
	
	/// GUI Management tools.
	/** HTML5 on-canvas controls. */
	namespace GUI
	{
		/// Clickable button.
		/** This is just a clickable button. */
		public class Button
		{
			/// Coordinate X.
			/** The X coordinate on the canvas of the button (from left). */
			public var x;
			
			/// Coordinate Y.
			/** The Y coordinate on the canvas of the button (from top). */
			public var y;
			
			/// Width of the button.
			/** Width of the button in pixels. */
			public var width;
			
			/// Height of the button.
			/** Height of the button in pixels. */
			public var height;
			
			/// Text in the button.
			/** The text that the button contains. */
			public var text;
			
			/// Defines if button is interactable.
			/** If enabled the button can be clicked, else it won't respond to input. */
			public var enabled;
			
			/// Create a button.
			/** Create a button that can be rendered on the canvas.
			  *
			  * @code
			  * var btnSimpleButton = new GameEngine.GUI.Button(10, 10, 300, 30, "Click me!");
			  * @endcode
			  *
			  * @param x The X coordinate of button on canvas.
			  * @param y The Y coordinate of button on canvas.
			  * @param width The width of the button in pixels.
			  * @param height The height of the button in pixels.
			  * @param text The text inside the button.
			  */
			public Button(var x, var y, var width, var height, var text) {}
			
			/// Draw the button.
			/** The button gets drawn on the canvas (automatically manages mouse events).
			  * 
			  * @return true if button is clicked, false otherwise.
			  */
			public render() {}
		}
		
		/// Selectable checkbox.
		/** This is a selectable checkbox. */
		public class CheckBox
		{
			/// @todo doc not written.
		}
		
		/// Text label.
		/** This is a label that contains a string of text. */
		public class Label
		{
			/// @todo doc not written.
		}
	}
	
	/// Load media and start game.
	/** Place this after setting all your objects, gameFunction will be called continuously as all objects will be loaded.<br>
	  * This is useful to start executing the game only when everything is ready, this will prevent freezing to load additional media.
	  *
	  * @code
	  * var testModel1 = new GameEngine.Renderer.Model("test1.png");
	  * var testModel2 = new GameEngine.Renderer.Model("test2.png");
	  * 
	  * function gameFunction() {
	  * 	GameEngine.Renderer.clearScreen();
	  * 
	  * 	testModel1.render();
	  * 	testModel2.render();
	  * 
	  * 	testModel2.x = testModel1.x;
	  * 	testModel2.y = testModel1.y;
	  * 
	  * 	testModel1.x = Math.random()*GameEngine.Renderer.canvas.width;
	  * 	testModel1.y = Math.random()*GameEngine.Renderer.canvas.height;
	  * }
	  * 
	  * GameEngine.startGame(gameFunction);
	  * @endcode
	  *
	  * @param gameFunction A function to be called continuously (every frame).
	  */
	public startGame(var gameFunction) {}
}